unit plugin;
             // image processing demo plugin for MSB Astroart (version 8.0 SP4 or later)
interface

uses Windows, SysUtils;

function pi_initialize(funcaddress:Pointer; menuname,description:PAnsiChar; whandle:HWND):Integer; stdcall;
procedure pi_dialog(pimage:Pointer; whandle:HWND); stdcall;
procedure pi_finalize(whandle:HWND); stdcall;
procedure pi_process(pimage:Pointer; whandle:HWND); stdcall;

implementation

const
 PC_GREEN = 1;
 PC_RED = 2;
 PC_BLUE = 3;

 PD_PREVIEW = 1;
 PD_PREVIEW_REALTIME = 2;

type

TAstroartCommand = (ac_getbuffer, ac_getsize, ac_redraw, ac_close, ac_savefits,
  ac_copy, ac_writeheader, ac_readheader, ac_prepareundo, ac_thresholds,
  ac_transferfunc, ac_getselpoint, ac_getselpoint2, ac_gethwnd, ac_create,
  ac_open, ac_getimage, ac_getversion, ac_rename, ac_getname, ac_getra,
  ac_getdec, ac_getx, ac_gety, ac_modified,
  ac_blink, ac_flip, ac_resize, ac_findcoordinates, ac_reserved_1, ac_selectpoint,
  ac_selectrect, ac_reserved_2, ac_reserved_3, ac_reserved_4, ac_centeratlas,
  ac_displayhelp, ac_save8bit, ac_preprocessing, ac_executemacro, ac_showdialog,
  ac_getdialogparam, ac_getundobuffer
);

TFloatArray = array[0..1] of Single;
PFloatArray = ^TFloatArray;
TCallBackFunc = function(command:TAstroartCommand; p1,p2,p3:Pointer): Integer; stdcall;  // ): NativeInt for 64 bits


var
 callback: TCallBackFunc;

function pi_initialize(funcaddress:pointer; menuname,description:pansichar; whandle: HWND):integer; stdcall;
begin
  callback := funcaddress;
  strcopy(menuname,'SDK demo plugin');
  strcopy(description,'Demo plugin in Pascal - a high pass filter');
  result := 1;
end;

procedure pi_finalize(whandle:HWND); stdcall;
begin
  // optional clean-up
end;

const
 DLG_NAME: AnsiString = 'SDK Demo';

procedure pi_dialog(pimage:pointer; whandle:HWND); stdcall;
var
 flags: Integer;
 s: AnsiString;
begin
  s := 'NUMBER "High pass" 70 0 100 5"'#13 +
       'BOOLEAN "Flip image"  0'#13;
  flags := PD_PREVIEW + PD_PREVIEW_REALTIME;
  callback(ac_showdialog, PAnsiChar(DLG_NAME), PAnsiChar(s), Pointer(flags));
end;

var
 dimx,dimy: Integer;
 imagebuffer,undobuffer: PFloatArray;

function getPixel(x,y:Integer): Single;
begin
  if x < 0 then x := 0;
  if y < 0 then y := 0;
  if x >= dimx then x := dimx-1;
  if y >= dimy then y := dimy-1;
  result := undobuffer[x+y*dimx];
end;

procedure setPixel(x,y:Integer; v:Single);
begin
  if (x<0) or (y<0) or (x >= dimx) or (y >= dimy) then exit;
  imagebuffer[x+y*dimx] := v;
end;

procedure pi_process(pimage:pointer; whandle:HWND); stdcall;
var
 v1,v2,vnew,vori,c1,c2: Double;
 ix,iy,x,y,b: Integer;
 cflip: Boolean;
begin
  v1 := PDouble(callback(ac_getdialogparam, PAnsiChar(DLG_NAME), Pointer(1), nil))^;
  c1 := v1 / 100.0;
  c2 := 1.0 - c1;
  v2 := PDouble(callback(ac_getdialogparam, PAnsiChar(DLG_NAME), Pointer(2), nil))^;
  cflip := v2 <> 0;
  callback(ac_getsize, pimage, @dimx, @dimy);

  b := callback(ac_getbuffer, pimage, @imagebuffer, pointer(PC_GREEN));
  if b <> 2 then exit;
  b := callback(ac_getundobuffer, pimage, @undobuffer, pointer(PC_GREEN));
  if b <> 2 then exit;
  for iy := 0 to dimy-1 do begin
     y := iy;
     if cflip then
        y := dimy-1-iy;
     for ix := 0 to dimx-1 do begin
        x := ix;
        vnew := getPixel(x,y)*5.0 - getPixel(x+1,y) - getPixel(x,y+1) - getPixel(x-1,y) - getPixel(x,y-1);
        vori := getPixel(x,y);
        setPixel(ix, iy, vnew*c1 + vori*c2);
     end;
  end;
  // on color images you should repeat this block for the other two color planes,
  // (PC_RED, PC_BLUE), otherwise only the green plane will be processed.
  // You could use a cycle: for i = PC_GREEN to PC_BLUE ...
  // To check if an image is grayscale compare the pointer of PC_GREEN to PC_RED or PC_BLUE,
  // if they are identical then the image is grayscale.
  // e.g.
  // callback(ac_getbuffer, pimage, @greenbuffer, pointer(PC_GREEN));
  // callback(ac_getbuffer, pimage, @redbuffer, pointer(PC_RED));
  // if greenbuffer = redbuffer then the image is grayscale
end;


end.

