unit plugin;
               // Plugin demo for MSB Astroart (version 5.0 or later)
interface

uses Windows, SysUtils;

function pi_initialize(funcaddress:pointer; menuname,description:pchar;
                      whandle:HWND):integer; stdcall;
procedure pi_activate(pimage:pointer; whandle:HWND); stdcall;
procedure pi_finalize(whandle:HWND); stdcall;

implementation

const
 PACTIVEIMAGE = 4096;
 PSELECTIMAGE = 4097;
 PEQU_STD = 1000;
 PEQU_GAU = 1001;
 PCGREEN = 0;
 PCRED = 2;
 BCBLUE = 3;

type

TAstroartCommand = (ac_getbuffer, ac_getsize, ac_redraw, ac_close, ac_savefits,
  ac_copy, ac_writeheader, ac_readheader, ac_prepareundo, ac_thresholds,
  ac_transferfunc, ac_getselpoint, ac_getselpoint2, ac_gethwnd, ac_create,
  ac_open, ac_getimage, ac_getversion, ac_rename, ac_getname, ac_getra,
  ac_getdec, ac_getx, ac_gety, ac_modified,
  ac_blink, ac_flip, ac_resize, ac_findcoordinates, ac_reserved_1, ac_selectpoint,
  ac_selectrect, ac_reserved_2, ac_reserved_3, ac_reserved_4, ac_centeratlas
);

T_FloatArray = array[0..1] of single;
pfloatarray = ^T_FloatArray;
Tcallbackfunc = function(command:TAstroartCommand; p1,p2,p3:pointer):integer; stdcall;

(* Commands: see docs

NAME             PARAMS TYPE                   RESULT

ac_getbuffer    (pimage, &buffer, colorplane)  0,2
ac_getsize      (pimage, &x, &y)               0,1
ac_redraw       (pimage, calcstats, autovis)   0,1
ac_close        (pimage, askuser, nil)         0,1
ac_savefits     (pimage, pfilename, nil)       0,1
ac_copy         (pimage, nil, nil)             0,1
ac_writeheader  (pimage, pheader, nil)         0,1
ac_readheader   (pimage, nil, nil)             pheader
ac_prepareundo  (pimage, nil, nil)             0,1
ac_thresholds   (pimage, &min, &max)           0,1
ac_transferfunc (pimage, trf, nil)             0,1
ac_getselpoint  (pimage, &x, &y)               0,1
ac_getselpoint2 (pimage, &x, &y)               0,1
ac_gethwnd      (pimage, &HWND, nil)           0,1
ac_create       (x,y,colored)                  pimage
ac_open         (pfilename, nil, nil)          pimage
ac_getimage     (selection, nil, nil)          pimage

ac_getversion   (nil, nil, nil)                300+
ac_rename       (pimage, name, path)           0,1
ac_getname      (pimage, path, nil)            pfilename
ac_getra        (pimage, x*100, y*100)         ra*1e5
ac_getdec       (pimage, x*100, y*100)         dec*1e5
ac_getx         (pimage, ra*1e6, dec*1e6)      x*100
ac_gety         (pimage, ra*1e6, dec*1e6)      y*100
ac_modified     (pimage, modified, nil)        0,1

ac_blink        (NULL, NULL, NULL)              0,1
ac_flip         (pimage, axis, NULL)            0,1
ac_resize       (pimage, dimx, dimy)            0,1
ac_findcoordinates (pimage, pparameters, NULL)  0,1
ac_selectpoint  (pimage, x, y)                  0,1
ac_selectrect   (pimage, prect, NULL)           0,1
ac_centeratlas  (ra*1E6, dec*1E6, showAtlas)    1
*)


var
 callback: Tcallbackfunc;
 dimx,dimy: integer;
 newbuffer,oldbuffer: pfloatarray;


// Utility functions. Not optimized, for learning purposes.

function getPixel(x,y: integer): single;
begin
   if x < 0 then x := 0;
   if y < 0 then y := 0;
   if x >= dimx then x := dimx-1;
   if y >= dimy then y := dimy-1;
   result := oldbuffer[x+y*dimx];
end;

procedure setPixel(x,y: integer; v:single);
begin
   if (x<0) or (y<0) or (x >= dimx) or (y >= dimy) then exit;
   newbuffer[x+y*dimx] := v;
end;


// Functions exported by the DLL

function pi_initialize(funcaddress:pointer; menuname,description:pchar;
         whandle: HWND):integer; stdcall;
begin
   callback := funcaddress;
   strcopy(menuname,'Delphi hi-pass demo');
   strcopy(description,'Pascal Plug-in for Astroart 4.0/5.0 - a high pass filter');
   result := 1;
end;

procedure pi_finalize(whandle:HWND); stdcall;
begin
   // optional clean-up
end;

procedure pi_activate(pimage:pointer; whandle:HWND); stdcall;
var
 x,y,b: integer;
 v: single;
begin
   if pimage = nil then begin
      messagebox(0,'ERROR','No image !',0);   // no images opened
      exit;
   end;
   callback(ac_getsize,pimage,@dimx,@dimy);               // get image size
   callback(ac_prepareundo,pimage,nil,nil);
   getmem(oldbuffer,dimx*dimy*4);

   b := callback(ac_getbuffer,pimage,@newbuffer,pointer(PCGREEN));  //***
   if b <> 2 then exit;     // buffer is not 32 bit, old version of Astroart
   copymemory(oldbuffer,newbuffer,dimx*dimy*4); // copy image to  an another buffer
   for y := 0 to dimy-1 do                    // convolution mask:     0 -1  0
    for x := 0 to dimx-1 do                                      //   -1  6 -1
    begin                                                        //    0 -1  0
       v := (getPixel(x,y)*6 - getPixel(x+1,y) - getPixel(x,y+1)
            - getPixel(x-1,y) - getPixel(x,y-1)) / 2;        // 2 = 6-1-1-1-1
       setPixel(x,y,v);
    end;
   //*** on color images you should repeat these steps for other color planes.

   freemem(oldbuffer);
   callback(ac_redraw,pimage,pointer(1),pointer(0));  // update the filtered image
end;


end.

